# Boot Options

## ilitools_profile

Pre-seed the profile to use if we're auto-booting the installer. This can be a profile name, disk path or URL starting with "http".


# Configuration


## ALL targets

### Option: **data_search_path**

**Valid in**: commandline

Specify search paths using comma separated format. If you're running from the project directory, you can specify ".".


### Option: **staging**

**Valid in**: profile, commandline

Use staging repositories, this sets the default for `pacman_conf` below.


### Option: **pacman_conf**

**Valid in**: profile, commandline

Specify a pacman configuration file directly, this will override the default set above.


### Option: **packages**

**Valid in**: profile, commandline, prompt

List of additional packages to install, either a list or comma-separated, or both.


### Option: **services**

**Valid in**: profile, commandline, prompt

List of services to enable automatically, either a list or comma-separated, or both.


### Option: **hostname**

**Valid in**: profile, commandline, prompt

Hostname to use for the system. This defaults to `localhost.localdomain`.


### Option: **root_password**

**Valid in**: profile, prompt

Root password to be used.

A blank password would result in the root account being locked.

A single space in the password field would cause the root account to login without password.


### Option: **sudo_username**

**Valid in**: profile, prompt

Optional second user account to create which has sudo permissions to root.


### Option: **sudo_password**

**Valid in**: profile, prompt

Password of the second user account. A blank password would result in the account being locked.

A single space in the password field would cause the account to login without password.


### Option: **root_autologin**

**Valid in**: profile, commandline

Enable autologin to root during startup, this can be `True` or `False`.

For the commandline the option is split into two `enable-root-autologin` and `disable-root-autologin`.


### Option: **overlay_directory**

**Valid in**: profile, commandline

Overlay directory to add to the image we're generating. This is in the format of `/host/path/to/dir:/target/path`.

This is either a list or comma-separated, or both.


### Option: **install_microcode**

**Valid in**: profile, commandline

Enable or disable the installation of microcode. Enabled by default for `disk-image` and `live-image`.


### Option: **networking**

**Valid in**: profile, commandline

From the commandline this is called `--disable-networking`.

Enable or disable networking by specifying a `True` or `False` value.


### Option: **network_devices**

**Valid in**: profile, commandline

List of devices to configure, either a list or comma-separated, or both.

The following additional profile options can be specified to set defaults...

* network_device_{device_name} - DHCP, NONE or STATIC
* network_device_{device_name}_mac - MAC address of interface, naming will be forced if specified
* network_device_{device_name}_alias - Network interface alias
* network_device_{device_name}_description - Description of interface
* network_device_{device_name}_ipv4address - IPv4 address of interface in CIDR format
* network_device_{device_name}_ipv6address - IPv6 address of interface in CIDR format
* network_device_{device_name}_ipv4dns - IPv4 DNS servers, comma-separated or list
* network_device_{device_name}_ipv6dns - IPv6 DNS servers, comma-separated or list
* network_device_{device_name}_ipv4gateway - IPv4 gateway
* network_device_{device_name}_ipv6gateway - IPv6 gateway



## Target: container-dir

The `container-dir` target is used for writing a container to a directory. This can be used for chroots, LXC and systemd-nspawn.

### Option: **target_dir**

**Valid in**: profile, commandline, prompt

This is where the container will be written to.



## Target: container-tar

The `container-tar` target is used for creating a container and tar'ing it up into an archive.

### Option: **target_file**

**Valid in**: profile, commandline, prompt

File target. The target file can have the following extensions: `tar`, `tar.gz`, `tar.bz2`,  `tar.xz`, `tar.zst`



## Target: disk-image

This target creates a disk image which can be written directly to disk.


### Option: **target_image**

**Valid in**: profile, commandline, prompt

File target. The target image can have the following extensions: `raw`, `qcow2`


### Option: **image_size**

**Valid in**: profile, commandline, prompt

Size of the resulting target image. This should be specified in string format, eg. `20G`. Defaults to `4G`.


### Option: **disk_label**

**Valid in**: profile, commandline

Disk label to use for the VM image. Either `dos` or `gpt`. Defaults to `gpt`.


### Option: **create_efi**

**Valid in**: profile, commandline

Install an EFI stub to boot the image via EFI. The resulting `/efi` partition will be 64MiB in size.

The stub will bootstrap to /boot.

This disabled by default.

Using this option from the commandline, one would use `--enable-efi`.


### Option: **create_bootfs**

**Valid in**: profile, commandline

Create a separate `/boot` filesystem. By default this will be 500MiB in size.

This disabled by default.

Using this option from the commandline, one would use `--enable-bootfs`.

The boot filesystem will be disabled if the filesystem is "btrfs" to provide for better full system snapshots.


### Option: **rootfs_type**

**Valid in**: profile, commandline

Specify the root filesystem type to use.

This option defaults to 'ext4'.

Using this option from the commandline, one would use `--rootfs-type`.


### Option: **autoresize_root**

**Valid in**: profile, commandline

Disable auto-resize of rootfs.

Using this option from the commandline, one would use `--disable-autoresize-root`.

This option is not supported for encrypted volumes.


### Option: **install_microcode**

**Valid in**: profile, commandline

Enable or disable the use of microcode updates.

This defaults to being disabled. Set to any true value to enable.

Using this option from the commandline, one would use `--enable-microcode-update`.


### Option: **kernel**

**Valid in**: profile, commandline

Kernel to use, the supported options are: `linux`, `linux-lts`, `linux-hardened`. Defaults to `linux-lts`.


### Option: **serial_terminal**

**Valid in**: profile, commandline

Set the serial terminal to use. Defaults to none.


### Option: **serial_terminal_type**

**Valid in**: profile, commandline

Set the serial terminal type to use. Defaults to None (automatic).

Valued values are the same as TERM=, like `vt220`, `vt100` or `linux`.


### Option: **mkinitcpio_modules**

**Valid in**: profile, commandline

Modules to add to the initramfs. This is only useful if mkinitcpio autodetect is disabled.


### Option: **mkinitcpio_autodetect**

**Valid in**: profile, commandline

Enable auto-detection of kernel modules to use. This will result in only the kernel modules for the current hardware being used.

By default this option is disabled.

This is probably not what you want!

Using this option from the commandline, one would use `--enable-mkinitcpio-autodetect`.


### Option: **use_cloudinit**

**Valid in**: profile, commandline

Enable or disable the use of cloud-init. By default cloud-init is enabled.

Using this option from the commandline, one would use `--disable-cloudinit`.


### Option: **cloudinit_enable_root**

**Valid in**: profile, commandline

Enable root login when using cloud-init (disabled by default).


### Option: **cloudinit_admin_username**

**Valid in**: profile, commandline

Set the admin username created by cloud-init default configuration.


### Option: **sshd_client_alive_interval**

**Valid in**: profile, commandline

Set the value of ClientAliveInterval in /etc/ssh/sshd_config.



## Target: disk-install

This target installs directly onto a disk or disks.


### Option: **target_disks**

**Valid in**: profile, commandline, prompt

Comma separated list of target disks to install on.


### Option: **disk_usage_strategy***

**Valid in**: profile, commandline, prompt

Disk usage strategy to use, eg. ASIS, MDRAID1, MDRAID5, MDRAID6, MDRAID10


### Option: **disk_layout_strategy***

**Valid in**: profile, commandline, prompt

Disk layout strategy to use, eg. FLAT, LVM


### Option: **disk_encryption_strategy***

**Valid in**: profile, commandline, prompt

Disk layout strategy to use, eg. LUKS-ON-PART, LVM-ON-LUKS

For the above the the associated layout strategy must be used, for ASIS use LUKS-ON-PART, for LVM use LVM-ON-LUKS.


### Option: **disk_encryption_password***

**Valid in**: profile, prompt

Disk encryption password to use.


### Option: **network_device_naming_strategy***

**Valid in**: profile, commandline, prompt

Network device naming strategy. eg. ASIS, AUTO-SEQ


### Option: **create_efi**

**Valid in**: profile, commandline

Install an EFI stub to boot the image via EFI. The resulting `/efi` partition will be 64MiB in size.

The stub will bootstrap to /boot.

This is enabled by default, to disable set to a false value.

Using this option from the commandline, one would use `--disable-efi`.


### Option: **create_bootfs**

**Valid in**: profile, commandline

Create a separate `/boot` filesystem. By default this will be 500MiB in size.

This is created by default, to disable set to a false value.

Using this option from the commandline, one would use `--disable-bootfs`.

The boot filesystem will be disabled if the filesystem is "btrfs" and disk encryption is not used to ensure better full system
snapshots.


### Option: **rootfs_type**

**Valid in**: profile, commandline

Specify the root filesystem type to use.

This option defaults to 'ext4'.

Using this option from the commandline, one would use `--rootfs-type`.


### Option: **autoresize_root**

**Valid in**: profile, commandline

Auto-resizing of rootfs.

This option is disabled by default, set to a true value to enable.

Using this option from the commandline, one would use `--enable-autoresize-root`.

### Option: **install_microcode**

**Valid in**: profile, commandline

Enable or disable the use of microcode updates.

This defaults to being enabled. Set to any false value to disable.

Using this option from the commandline, one would use `--disable-microcode-update`.


### Option: **kernel**

**Valid in**: profile, commandline

Kernel to use, the supported options are: `linux`, `linux-lts`, `linux-hardened`. Defaults to `linux-lts`.


### Option: **serial_terminal**

**Valid in**: profile, commandline

Set the serial terminal to use. Defaults to what is found on the kernel command line.


### Option: **mkinitcpio_modules**

**Valid in**: profile, commandline

Modules to add to the initramfs. This is only useful if mkinitcpio autodetect is disabled.


### Option: **mkinitcpio_autodetect**

**Valid in**: profile, commandline

Used to auto-detect kernel modules to use. This will result in only the kernel modules for the current hardware being used.

By default this option is enabled.

Using this option from the commandline, one would use `--disable-mkinitcpio-autodetect`.


### Option: **use_cloudinit**

**Valid in**: profile, commandline

Enable or disable the use of cloud-init. By default cloud-init is disabled.

Using this option from the commandline, one would use `--enable-cloudinit`.


### Option: **cloudinit_enable_root**

**Valid in**: profile, commandline

Enable root login when using cloud-init (disabled by default).


### Option: **cloudinit_admin_username**

**Valid in**: profile, commandline

Set the admin username created by cloud-init default configuration.



## Target: live-image

This target creates a bootable live image.


### Option: **target_file**

**Valid in**: profile, commandline, prompt

Target filename.


### Option: **kernel**

**Valid in**: profile, commandline

Kernel to use, the supported options are: `linux`, `linux-lts`, `linux-hardened`. Defaults to `linux-lts`.



### Autorun functionality

The below only applies when root auto-login is enabled which is the default on the live media.

The kernel commandline argument `autorun=` can be specified with a URL to download and run a script on startup. The URL can either
begin with `http://` or `https://`.

This feature also ties into the the Qemu VM functionality below in the listed order:
  * If the 9P `host` filesystem was mounted it will result in `/autorun` on that filesystem being automatically run if it exists.
  * If the `OEM` labelled filesystem was mounted it will result in `/autorun` on that being automatically run if it exists.

The order in which the `autorun` script is located is as follows:
  * kernel commandline
  * /OEM/autorun
  * /host/autorun



# Live image functionality

The live image supports a few additional capabilities.


## For OEM's

The Grub boot process can be overridden using an additional volume plugged into the system with partition label `OEM`
and a `/grub.cfg` file.

The configuration is sourced and the variable `$oem` is set to the device.

The `OEM` filesystem will be mounted as `/OEM` if it exists.


## Qemu VM: Host system OverlayFS

To use the host system as an OverlayFS backing storage, the `idms.live_cowspace_type=9p` can be passed as a kernel commandline
option either manually or using the OEM functionality above.

The live media will automatically mount the 9p filesystem mount tag `cowspace` and use this to back the live media root.

This can be used as either additional storage for software running within the live media environment or to track which files
are changed within the environment.

You can pass the `cowspace` filesystem to the virtual environment using the following...
```
-fsdev local,id=fsdev0,path='/path/to/folder',security_model=mapped-xattr -device virtio-9p-pci,fsdev=fsdev0,addr=04.0,mount_tag=cowspace
```


## Qemu VM: Host file sharing

If a 9p filesystem with mount tag `host` is exposed to the live media, it will be automatically mounted as `/host`.

You can do this in Qemu using the below additional command line options...
```
-fsdev local,id=fsdev0,path='/path/to/folder',security_model=mapped-xattr -device virtio-9p-pci,fsdev=fsdev0,addr=04.0,mount_tag=host
```


# Testing

## BIOS

Using normal BIOS booting and a CDROM...
```
qemu-system-x86_64 -machine q35,accel=kvm -m 2048 -cdrom live.img
```

Using normal BIOS booting and a USB stick...
```
qemu-system-x86_64 -machine q35,accel=kvm -m 2048 -drive if=none,format=raw,readonly,id=stick,file=live.img -device nec-usb-xhci,id=xhci -device usb-storage,bus=xhci.0,drive=stick
```

Using normal booting and a HDD...
```
qemu-system-x86_64 -machine q35,accel=kvm -m 2048 -hda live.img
```

## EFI

Using EFI booting and a CDROM...
```
qemu-system-x86_64 -machine q35,accel=kvm -m 2048 -bios /usr/share/ovmf/x64/OVMF_CODE.fd -cdrom live.img
```

Using EFI booting and a USB stick...
```
qemu-system-x86_64 -machine q35,accel=kvm -m 2048 -bios /usr/share/ovmf/x64/OVMF_CODE.fd -drive if=none,format=raw,readonly,id=stick,file=live.img -device nec-usb-xhci,id=xhci -device usb-storage,bus=xhci.0,drive=stick,bootindex=0
```

Using EFI booting and a HDD...
```
qemu-system-x86_64 -machine q35,accel=kvm -m 2048 -bios /usr/share/ovmf/x64/OVMF_CODE.fd -hda live.img
```


## Image Comparison Matrix

| Flavor/Feature  | aws                      | azure                    | qemu                     | qemu-efi                 | raw-efi                  | raw-efi-serial           |
| :----:          | :----:                   | :----:                   | :----:                   | :----:                   | :----:                   | :----:                   |
| Format          | raw                      | vhd                      | qcow2                    | qcow2                    | raw                      | raw                      |
| Special         | :heavy_multiplication_x: | :heavy_check_mark:       | :heavy_multiplication_x: | :heavy_multiplication_x: | :heavy_multiplication_x: | :heavy_multiplication_x: |
| BIOS/EFI        | BIOS                     | BIOS                     | BIOS                     | BIOS+EFI                 | BIOS+EFI                 | BIOS+EFI                 |
| Initial size    | 4G                       | 30G                      | 4G                       | 4G                       | 4G                       | 4G                       |
| Auto resize     | :heavy_check_mark:       | :heavy_check_mark:       | :heavy_check_mark:       | :heavy_check_mark:       | :heavy_check_mark:       | :heavy_check_mark:       |
| Serial          | :heavy_check_mark:       | :heavy_check_mark:       | :heavy_multiplication_x: | :heavy_multiplication_x: | :heavy_multiplication_x: | :heavy_check_mark:       |
| Guest agent     | none                     | walinuxagent             | qemu-guest-agent         | qemu-guest-agent         | none                     | none                     |
| Cloud-Init      | :heavy_check_mark:       | :heavy_multiplication_x: | :heavy_check_mark:       | :heavy_check_mark:       | :heavy_check_mark:       | :heavy_check_mark:       |

TODO: Add section on serial support

## Container Comparison Matrix

| Flavor/Feature  | container                | docker             |
| :----:          | :----:                   | :----:             |
| Stripped Down   | :heavy_multiplication_x: | :heavy_check_mark: |


# Automated Install Testing

You can use my test profile to create a live image with the installer...
```sh
sudo python -m ilitools.commandline --verbose create --data-search-paths . --profile contrib/profiles/install-test.profile live-image --target-file ~/tmp/live.img
qemu-img create -f qcow2 ~/tmp/vm1.qcow2 10G
qemu-img create -f qcow2 ~/tmp/vm2.qcow2 10G
qemu-system-x86_64 -m 2048 -accel kvm -net nic -net bridge,br=br0 -cdrom ~/tmp/live.img -hda ~/tmp/vm1.qcow2 -hdb ~/tmp/vm2.qcow2
```

